#include <s_globals.h>
#include <s_util.h>
#include <sensor.h>
#include <cleanup.h>
#include <targets.h>
#include <loader.h>
#include <capdev.h>
#include <demux.h>
#include <mem.h>

/* Our one and only global structure */
struct sensor_globals_t globals;

void sig_proc(int code)
{
	switch (code)
	{
		case SIGHUP:
		case SIGTERM:
		case SIGINT:
			fprintf(stderr, CRLF "Got signal %i, "
				"scheduling cleanup" CRLF , code);
			globals.keepalive.counter=0;
			return;
	}
}

const char *plugin_get_var(const char *name)
{
	p_kv t;

	if ( !name ) return NULL;
	
	for(t=globals.plugin_vars; t; t=(p_kv)t->l.next) {
		if ( !strcasecmp(name, t->key) ) {
			return t->val;
		}
	}
	
	return NULL;
}

void *capture_proc(void *parm)
{
	p_capdev c;
	p_packet p;
	struct tm *tmpt;

	uint a=0;
	
	c=(p_capdev)parm;
	c->state=CAP_STATE_CAPTURE;

	while(1){

		/* Poll the capture device */
		if ( (p=c->capture->poll(c->priv)) == NULL ) break;
		
		/* Convert the time to local time*/
		CRITICAL_START(globals.tlock);
		tmpt = localtime(&p->time);
		memcpy(&p->ltime, tmpt, sizeof(p->ltime));
		CRITICAL_STOP(globals.tlock);
		
		/* Tell the packet who cought it */
		p->capdev=c;

		/* Demiltiplex... */
		demux(p);

		/* Match */
		matcher_begin(p);

		/* Per capdev packet count */
		a++;
	}

	print_out("Ending %s capture [%s]: Caught %u packets." CRLF, 
		c->capture->name, c->interface, a);
	
	c->capture->end(c->priv);
	c->state=CAP_STATE_NULL;
	
	atomic_dec_counter(&globals.keepalive);
	return;	
}

int main ( int argc, char **argv ) 
{
	memset(&globals, 0, sizeof(globals));
	tzset();
	
#ifdef LEAK_CHECK
	if ( !mem_init () ) return EXIT_ERR;
	globals.malloc=fs_alloc;
	globals.free=fs_free;
	globals.fs_strdup=fs_strdup;
#else
	globals.malloc=malloc;
	globals.free=free;
	globals.fs_strdup=strdup;
#endif

	print_raw(VERSION_STRING CRLF);

	if ( argc != 2 ) {
		print_raw("Usage: sensor <configfile>" CRLF);
		return EXIT_ERR;
	}else{
		globals.config_fn=argv[1];
	}
	
	/* Can't go nowhere with no cleanup subsystem */
	if ( !cleanup_init() ) return EXIT_ERR;
	
	/* Create global mutex, and add cleanup code */
	MUTEX_CREATE(globals.glock, "Global");
	MUTEX_CREATE(globals.tlock, "localtime()");
	MUTEX_CREATE(globals.keepalive.lock, "Keepalive");
	globals.keepalive.counter=0;
	cleanup_add_handler(cleanup_lock, (void *)&globals.glock);
	cleanup_add_handler(cleanup_lock, (void *)&globals.keepalive.lock);
	
	/* Set signal handlers */
	signal(SIGTERM, sig_proc);
	signal(SIGINT, sig_proc);
	signal(SIGHUP, sig_proc);

	
	/* Load capture subsystem */
	if ( !capdev_init() ) cleanup(EXIT_ERR);

	/* Load demultiplexing subsystem */
	if ( !demux_init() ) cleanup(EXIT_ERR);
	
	/* Load packet target subsystem */
	if ( !target_init() ) cleanup(EXIT_ERR);
	
	/* Load packet matcher subsystem  */
	if ( !matcher_init() ) cleanup(EXIT_ERR);
	
	/* Load ruleset file subsytem  */
	if ( !rules_init() ) cleanup(EXIT_ERR);

	/* Load and run our config */
	if ( !conf_load_config(globals.config_fn) ) {
		conf_cleanup_temp();
		cleanup(EXIT_ERR);
	}
	if ( !conf_go() ) {
		conf_cleanup_temp();
		cleanup(EXIT_ERR);
	}conf_cleanup_temp();

	/* Print a quick summary */
	print_out("PLUGINS: %u plugins loaded, "
		"%u matchers, "
		"%u targets." CRLF,
		globals.num_plugins,
		globals.num_matchers,
		globals.num_targets);
	
	print_out("RULES: %u rule files loaded, "
		"%u rules all together." CRLF,
		globals.num_rulefiles,
		globals.num_rules);
	
#ifdef LEAK_CHECK
	fs_usage();
#endif
	
	/* Now we start capturing for real */
	capdev_start_all();
	print_raw(CRLF);
	
	while(globals.keepalive.counter) {
		/* I guess we would do regular maintenance
		* crap here at some point  */
		sleep(1);
	}
	
	cleanup(EXIT_OK);
}
